-------------------------------------------------------------------------
--Created by Papaew
-------------------------------------------------------------------------

ffi.cdef[[
typedef struct { double x, y; } vec_2;
]]

vec2 = function(a,b)
	if type(a) == 'number' and type(b) == 'number' then
		return _vec2(a,b)
	end
end

_vec2 = {}
local mt = {
	__index = {
	area = function(a) return a.x*a.x + a.y*a.y end,
	unpack = function(a) return a.x, a.y end,
	set = function(a, x,y) a.x, a.y = x,y end,
	setX = function(a, x) a.x = x end,
	setY = function(a, y) a.y = y end,
	normalize = function(a)
					local len = 0
					len = a.x*a.x + a.y*a.y
					len = math.sqrt(len)
					return _vec2(a.x/len, a.y/len)
				end,
	setMag = function(a, len)
					local nlen = 0
					nlen = a.x*a.x + a.y*a.y
					nlen = math.sqrt(nlen)
					a:set(a.x/nlen, a.y/nlen)
					a:set(a.x*len, a.y*len)
				end,
	dist = function(a,b) return ((a.x-b.x)^2 + (a.y-b.y)^2)^0.5 end,
	scalar = function(a,b) return a.x*b.x + a.y*b.y end,
	copy = function(a) return _vec2(a.x, a.y) end,
	clamp = function(a, minx, miny, maxx, maxy)
					a.x = math.max(minx, math.min(a.x, maxx))
					a.y = math.max(miny, math.min(a.y, maxy))
				end,
	
	add = function(a, b)
				if type(b) == 'number' then
					a.x = a.x + b
					a.y = a.y + b
				elseif type(b) == 'vec2' then
					a.x = a.x + b.x
					a.y = a.y + b.y
				end
			end,
	sub = function(a, b)
				if type(b) == 'number' then
					a.x = a.x - b
					a.y = a.y - b
				elseif type(b) == 'vec2' then
					a.x = a.x - b.x
					a.y = a.y - b.y
				end
			end,
	mul = function(a, b)
				if type(b) == 'number' then
					a.x = a.x * b
					a.y = a.y * b
				elseif type(b) == 'vec2' then
					a.x = a.x * b.x
					a.y = a.y * b.y
				end
			end,
	div = function(a, b)
				if type(b) == 'number' then
					a.x = a.x / b
					a.y = a.y / b
				elseif type(b) == 'vec2' then
					a.x = a.x / b.x
					a.y = a.y / b.y
				end
			end,
	mod = function(a, b)
				if type(b) == 'number' then
					a.x = a.x % b
					a.y = a.y % b
				elseif type(b) == 'vec2' then
					a.x = a.x % b.x
					a.y = a.y % b.y
				end
			end,
	pow = function(a, b)
			a.x = a.x ^ b
			a.y = a.y ^ b
		end,
	},
	__add = function(a,b)
				if type(a) == 'number' and type(b) == 'vec2' then
					return _vec2(a+b.x, a+b.y)
				elseif type(a) == 'vec2' and type(b) == 'number' then
					return _vec2(a.x+b, a.y+b)
				elseif type(a) == 'vec2' and type(b) == 'vec2' then
					return _vec2(a.x+b.x, a.y+b.y)
				end
			end,
	__sub = function(a,b)
				if type(a) == 'number' and type(b) == 'vec2' then
					return _vec2(a-b.x, a-b.y)
				elseif type(a) == 'vec2' and type(b) == 'number' then
					return _vec2(a.x-b, a.y-b)
				elseif type(a) == 'vec2' and type(b) == 'vec2' then
					return _vec2(a.x-b.x, a.y-b.y)
				end
			end,	
	__mul = function(a,b)
				if type(a) == 'number' and type(b) == 'vec2' then
					return _vec2(a*b.x, a*b.y)
				elseif type(a) == 'vec2' and type(b) == 'number' then
					return _vec2(a.x*b, a.y*b)
				elseif type(a) == 'vec2' and type(b) == 'vec2' then
					return _vec2(a.x*b.x, a.y*b.y)
				end
			end,
	
	__div = function(a,b)
				if type(a) == 'number' and type(b) == 'vec2' then
					return _vec2(a/b.x, a/b.y)
				elseif type(a) == 'vec2' and type(b) == 'number' then
					return _vec2(a.x/b, a.y/b)
				elseif type(a) == 'vec2' and type(b) == 'vec2' then
					return _vec2(a.x/b.x, a.y/b.y)
				end
			end,
	
	__mod = function(a,b)
				if type(a) == 'number' and type(b) == 'vec2' then
					return _vec2(a%b.x, a%b.y)
				elseif type(a) == 'vec2' and type(b) == 'number' then
					return _vec2(a.x%b, a.y%b)
				elseif type(a) == 'vec2' and type(b) == 'vec2' then
					return _vec2(a.x%b.x, a.y%b.y)
				end
			end,
	
	__eq  = function(a,b)
				if type(a) == 'vec2' and type(b) == 'vec2' then
					return a.x==b.x and a.y==b.y
				else
					return false
				end
			end,
	__tostring = function(a) return 'vec2[x='..a.x..', y='..a.y..']' end,
	__unm = function(a) return _vec2(-a.x, -a.y) end,
	__len = function(a) return math.sqrt(a.x^2+a.y^2) end,
	__pow = function(a,b) return _vec2(a.x^b, a.y^b) end
}
_vec2 = ffi.metatype('vec_2', mt)