-------------------------------------------------------------------------
--Created by Papaew
-------------------------------------------------------------------------

ffi.cdef[[
typedef struct { double x, y, z; } vec_3;
]]

vec3 = function(a,b,c)
	if type(a) == 'number' and type(b) == 'number' and type(c) == 'number' then
		return _vec3(a,b,c)
	end
end

_vec3 = {}
local mt = {
	__index = {
	area = function(a) return a.x*a.x + a.y*a.y + a.z*a.z end,
	unpack = function(a) return a.x, a.y, a.z end,
	set = function(a, x,y,z) a.x, a.y, a.z = x,y,z end,
	setX = function(a, x) a.x = x end,
	setY = function(a, y) a.y = y end,
	setZ = function(a, z) a.z = z end,
	normalize = function(a)
					local len = 0
					len = a.x*a.x + a.y*a.y + a.z*a.z
					len = math.sqrt(len)
					return _vec3(a.x/len, a.y/len, a.z/len)
				end,
	setMag = function(a, len)
					local nlen = 0
					nlen = a.x*a.x + a.y*a.y + a.z*a.z
					nlen = math.sqrt(nlen)
					a:set(a.x/nlen, a.y/nlen, a.z/nlen)
					a:set(a.x*len, a.y*len, a.z*len)
				end,
	dist = function(a,b) return ((a.x-b.x)^2 + (a.y-b.y)^2 + (a.z-b.z)^2)^0.5 end,
	scalar = function(a,b) return a.x*b.x + a.y*b.y + a.z*b.z end,
	copy = function(a) return _vec3(a.x, a.y, a.z) end,
	clamp = function(a, minx, miny, maxx, maxy, minz, maxz)
					a.x = math.max(minx, math.min(a.x, maxx))
					a.y = math.max(miny, math.min(a.y, maxy))
					a.z = math.max(minz, math.min(a.z, maxz))
				end,
	
	add = function(a, b)
				if type(b) == 'number' then
					a.x = a.x + b
					a.y = a.y + b
					a.z = a.z + b
				elseif type(b) == 'vec3' then
					a.x = a.x + b.x
					a.y = a.y + b.y
					a.z = a.z + b.z
				end
			end,
	sub = function(a, b)
				if type(b) == 'number' then
					a.x = a.x - b
					a.y = a.y - b
					a.z = a.z - b
				elseif type(b) == 'vec3' then
					a.x = a.x - b.x
					a.y = a.y - b.y
					a.z = a.z - b.z
				end
			end,
	mul = function(a, b)
				if type(b) == 'number' then
					a.x = a.x * b
					a.y = a.y * b
					a.z = a.z * b
				elseif type(b) == 'vec3' then
					a.x = a.x * b.x
					a.y = a.y * b.y
					a.z = a.z * b.z
				end
			end,
	div = function(a, b)
				if type(b) == 'number' then
					a.x = a.x / b
					a.y = a.y / b
					a.z = a.z / b
				elseif type(b) == 'vec3' then
					a.x = a.x / b.x
					a.z = a.z / b.z
				end
			end,
	mod = function(a, b)
				if type(b) == 'number' then
					a.x = a.x % b
					a.y = a.y % b
					a.z = a.z % b
				elseif type(b) == 'vec3' then
					a.x = a.x % b.x
					a.y = a.y % b.y
					a.z = a.z % b.z
				end
			end,
	pow = function(a, b)
			a.x = a.x ^ b
			a.y = a.y ^ b
			a.z = a.z ^ b
		end,
	},
	__add = function(a,b)
				if type(a) == 'number' and type(b) == 'vec3' then
					return _vec3(a+b.x, a+b.y, a+b.z)
				elseif type(a) == 'vec3' and type(b) == 'number' then
					return _vec3(a.x+b, a.y+b, a.z+b)
				elseif type(a) == 'vec3' and type(b) == 'vec3' then
					return _vec3(a.x+b.x, a.y+b.y, a.z+b.z)
				end
			end,
	__sub = function(a,b)
				if type(a) == 'number' and type(b) == 'vec3' then
					return _vec3(a-b.x, a-b.y, a-b.z)
				elseif type(a) == 'vec3' and type(b) == 'number' then
					return _vec3(a.x-b, a.y-b, a.z-b)
				elseif type(a) == 'vec3' and type(b) == 'vec3' then
					return _vec3(a.x-b.x, a.y-b.y, a.z-b.z)
				end
			end,	
	__mul = function(a,b)
				if type(a) == 'number' and type(b) == 'vec3' then
					return _vec3(a*b.x, a*b.y, a*b.z)
				elseif type(a) == 'vec3' and type(b) == 'number' then
					return _vec3(a.x*b, a.y*b, a.z*b)
				elseif type(a) == 'vec3' and type(b) == 'vec3' then
					return _vec3(a.x*b.x, a.y*b.y, a.z*b.z)
				end
			end,
	
	__div = function(a,b)
				if type(a) == 'number' and type(b) == 'vec3' then
					return _vec3(a/b.x, a/b.y, a/b.z)
				elseif type(a) == 'vec3' and type(b) == 'number' then
					return _vec3(a.x/b, a.y/b, a.z/b)
				elseif type(a) == 'vec3' and type(b) == 'vec3' then
					return _vec3(a.x/b.x, a.y/b.y, a.z/b.z)
				end
			end,
	
	__mod = function(a,b)
				if type(a) == 'number' and type(b) == 'vec3' then
					return _vec3(a%b.x, a%b.y, a%b.z)
				elseif type(a) == 'vec3' and type(b) == 'number' then
					return _vec3(a.x%b, a.y%b, a.z%b)
				elseif type(a) == 'vec3' and type(b) == 'vec3' then
					return _vec3(a.x%b.x, a.y%b.y, a.z%b.z)
				end
			end,
	
	__eq  = function(a,b)
				if type(a) == 'vec3' and type(b) == 'vec3' then
					return a.x==b.x and a.y==b.y and a.z==b.z
				else
					return false
				end
			end,
	__tostring = function(a) return 'vec3[x='..a.x..', y='..a.y..', z='..a.z..']' end,
	__unm = function(a) return _vec3(-a.x, -a.y, -a.z) end,
	__len = function(a) return math.sqrt(a.x^2+a.y^2+a.z^2) end,
	__pow = function(a,b) return _vec3(a.x^b, a.y^b, a.z^b) end
}
_vec3 = ffi.metatype('vec_3', mt)