-------------------------------------------------------------------------
--Created by Papaew
-------------------------------------------------------------------------

ffi.cdef[[
typedef struct { double x, y, z, w; } vec_4;
]]

vec4 = function(a,b,c,d)
	if type(a) == 'number' and type(b) == 'number' and type(c) == 'number' and type(d) == 'number' then
		return _vec4(a,b,c,d)
	end
end

_vec4 = {}
local mt = {
	__index = {
	area = function(a) return a.x*a.x + a.y*a.y + a.z*a.z + a.w*a.w end,
	unpack = function(a) return a.x, a.y, a.z, a.w end,
	set = function(a, x,y,z,w) a.x, a.y, a.z, a.w = x,y,z,w end,
	setX = function(a, x) a.x = x end,
	setY = function(a, y) a.y = y end,
	setZ = function(a, z) a.z = z end,
	setW = function(a, w) a.w = w end,
	normalize = function(a)
					local len = 0
					len = a.x*a.x + a.y*a.y + a.z*a.z + a.w*a.w
					len = math.sqrt(len)
					return _vec4(a.x/len, a.y/len, a.z/len, a.w/len)
				end,
	setMag = function(a, len)
					local nlen = 0
					nlen = a.x*a.x + a.y*a.y + a.z*a.z + a.w*a.w
					nlen = math.sqrt(nlen)
					a:set(a.x/nlen, a.y/nlen, a.z/nlen, a.w/nlen)
					a:set(a.x*len, a.y*len, a.z*len, a.w*len)
				end,
	dist = function(a,b) return ((a.x-b.x)^2 + (a.y-b.y)^2 + (a.z-b.z)^2 + (a.w-b.w)^2)^0.5 end,
	scalar = function(a,b) return a.x*b.x + a.y*b.y + a.z*b.z + a.w*b.w end,
	copy = function(a) return _vec4(a.x, a.y, a.z, a.w) end,
	clamp = function(a, minx, miny, maxx, maxy, minz, maxz, minw, maxw)
					a.x = math.max(minx, math.min(a.x, maxx))
					a.y = math.max(miny, math.min(a.y, maxy))
					a.z = math.max(minz, math.min(a.z, maxz))
					a.w = math.max(minw, math.min(a.w, maxw))
				end,
	
	add = function(a, b)
				if type(b) == 'number' then
					a.x = a.x + b
					a.y = a.y + b
					a.z = a.z + b
					a.w = a.w + b
				elseif type(b) == 'vec4' then
					a.x = a.x + b.x
					a.y = a.y + b.y
					a.z = a.z + b.z
					a.w = a.w + b.w
				end
			end,
	sub = function(a, b)
				if type(b) == 'number' then
					a.x = a.x - b
					a.y = a.y - b
					a.z = a.z - b
					a.w = a.w - b
				elseif type(b) == 'vec4' then
					a.x = a.x - b.x
					a.y = a.y - b.y
					a.z = a.z - b.z
					a.w = a.w - b.w
				end
			end,
	mul = function(a, b)
				if type(b) == 'number' then
					a.x = a.x * b
					a.y = a.y * b
					a.z = a.z * b
					a.w = a.w * b
				elseif type(b) == 'vec4' then
					a.x = a.x * b.x
					a.y = a.y * b.y
					a.z = a.z * b.z
					a.w = a.w * b.w
				end
			end,
	div = function(a, b)
				if type(b) == 'number' then
					a.x = a.x / b
					a.y = a.y / b
					a.z = a.z / b
					a.w = a.w / b
				elseif type(b) == 'vec4' then
					a.x = a.x / b.x
					a.z = a.z / b.z
					a.w = a.w / b.w
				end
			end,
	mod = function(a, b)
				if type(b) == 'number' then
					a.x = a.x % b
					a.y = a.y % b
					a.z = a.z % b
					a.w = a.w % b
				elseif type(b) == 'vec4' then
					a.x = a.x % b.x
					a.y = a.y % b.y
					a.z = a.z % b.z
					a.w = a.w % b.w
				end
			end,
	pow = function(a, b)
			a.x = a.x ^ b
			a.y = a.y ^ b
			a.z = a.z ^ b
			a.w = a.w ^ b
		end,
	},
	__add = function(a,b)
				if type(a) == 'number' and type(b) == 'vec4' then
					return _vec4(a+b.x, a+b.y, a+b.z, a+b.w)
				elseif type(a) == 'vec4' and type(b) == 'number' then
					return _vec4(a.x+b, a.y+b, a.z+b, a.w+b)
				elseif type(a) == 'vec4' and type(b) == 'vec4' then
					return _vec4(a.x+b.x, a.y+b.y, a.z+b.z, a.w+b.w)
				end
			end,
	__sub = function(a,b)
				if type(a) == 'number' and type(b) == 'vec4' then
					return _vec4(a-b.x, a-b.y, a-b.z, a-b.w)
				elseif type(a) == 'vec4' and type(b) == 'number' then
					return _vec4(a.x-b, a.y-b, a.z-b, a.w-b)
				elseif type(a) == 'vec4' and type(b) == 'vec4' then
					return _vec4(a.x-b.x, a.y-b.y, a.z-b.z, a.w-b.w)
				end
			end,	
	__mul = function(a,b)
				if type(a) == 'number' and type(b) == 'vec4' then
					return _vec4(a*b.x, a*b.y, a*b.z, a*b.w)
				elseif type(a) == 'vec4' and type(b) == 'number' then
					return _vec4(a.x*b, a.y*b, a.z*b, a.w*b)
				elseif type(a) == 'vec4' and type(b) == 'vec4' then
					return _vec4(a.x*b.x, a.y*b.y, a.z*b.z, a.w*b.w)
				end
			end,
	
	__div = function(a,b)
				if type(a) == 'number' and type(b) == 'vec4' then
					return _vec4(a/b.x, a/b.y, a/b.z, a/b.w)
				elseif type(a) == 'vec4' and type(b) == 'number' then
					return _vec4(a.x/b, a.y/b, a.z/b, a.w/b)
				elseif type(a) == 'vec4' and type(b) == 'vec4' then
					return _vec4(a.x/b.x, a.y/b.y, a.z/b.z, a.w/b.w)
				end
			end,
	
	__mod = function(a,b)
				if type(a) == 'number' and type(b) == 'vec4' then
					return _vec4(a%b.x, a%b.y, a%b.z, a%b.w)
				elseif type(a) == 'vec4' and type(b) == 'number' then
					return _vec4(a.x%b, a.y%b, a.z%b, a.w%b)
				elseif type(a) == 'vec4' and type(b) == 'vec4' then
					return _vec4(a.x%b.x, a.y%b.y, a.z%b.z, a.w%b.w)
				end
			end,
	
	__eq  = function(a,b)
				if type(a) == 'vec4' and type(b) == 'vec4' then
					return a.x==b.x and a.y==b.y and a.z==b.z and a.w==b.w
				else
					return false
				end
			end,
	__tostring = function(a) return 'vec4[x='..a.x..', y='..a.y..', z='..a.z..', w='..a.w..']' end,
	__unm = function(a) return _vec4(-a.x, -a.y, -a.z, -a.w) end,
	__len = function(a) return math.sqrt(a.x^2+a.y^2+a.z^2+a.w^2) end,
	__pow = function(a,b) return _vec4(a.x^b, a.y^b, a.z^b, a.w^b) end
}
_vec4 = ffi.metatype('vec_4', mt)