require "geometry"

shdr = [[
uniform mat4 view;
uniform mat4 model;
uniform mat4 proj;
uniform mat4 MVP;

#ifdef VERTEX
vec4 position(mat4 transform_projection, vec4 vertex_position) {

	return proj * view * model * vertex_position;
}
#endif

#ifdef PIXEL
vec4 effect(vec4 color, Image texture, vec2 texture_coords, vec2 screen_coords) {
	vec4 texturecolor = Texel(texture, texture_coords);
	return color*texturecolor;
	//return vec4(1,0,0,1);
}
#endif
]]

local width, height = love.graphics.getDimensions()
local cos,sin,tan,rad = math.cos, math.sin, math.tan, math.rad
function love.load()

	format = {{"VertexPosition", "float", 3}, {"VertexTexCoord", "float", 2}}
	vertices = {{-1,1,1, 0,1}, {1,1,1, 1,1}, {1,-1,1, 1,0}, {1,-1,1, 1,0}, {-1,-1,1, 0,0}, {-1,1,1, 0,1}}
	mesh = love.graphics.newMesh(format, vertices, "triangles", "static")
	texture = love.graphics.newImage("/pisi.png")
	mesh:setTexture(texture)
	shader = love.graphics.newShader(shdr)
	
	model = translate(0,0,0) * (rotateX(0)*rotateY(0)*rotateZ(0)) * scale(1,1,1)
	view = translate(0,0,0) * (rotateX(0)*rotateY(0)*rotateZ(0))
	projection = perspective_projection(90, width/height, 0.001, 100)
	
	shader:send("model", "row", model:unpack())
	shader:send("view", "row", view:unpack())
	shader:send("proj", "row", projection:unpack())
	
end

function love.update(dt)
	
end

function love.draw()
	love.graphics.setShader(shader)
	
	love.graphics.draw(mesh)

	love.graphics.setShader()
end


function rotateX(a)
return mat4(1, 0, 0, 0,
			0, cos(a), -sin(a), 0,
			0, sin(a), cos(a), 0,
			0, 0, 0, 1)
end

function rotateY(a)
return mat4(cos(a), 0, sin(a), 0,
			0, 1, 0, 0,
			-sin(a), 0, cos(a), 0,
			0, 0, 0, 1)
end

function rotateZ(a)
return mat4(cos(a),-sin(a), 0,	0,
			sin(a), cos(a), 0,	0,
			0, 0, 1, 0,
			0, 0, 0, 1)
end

function translate(x,y,z)
return mat4(1, 0, 0, x,
			0, 1, 0, y,
			0, 0, 1, z,
			0, 0, 0, 1)
end

function scale(x,y,z)
return mat4(x, 0, 0, 0,
			0, y, 0, 0,
			0, 0, z, 0,
			0, 0, 0, 1)
end

--[[function perspective_projection(fov, aspect, near, far)
local t = tan(fov/2)
return mat4(1/(aspect*t),0,0,0,
			0,1/t,0,0,
			0,0,(far+near)/(far-near),-1,
			0,0,-2*far*near/(far-near),0)
end]]

function perspective_projection(fov, aspect, near, far)
local t = tan(rad(fov)/2)
return mat4(1/(aspect*t),0,0,0,
			0,1/t,0,0,
			0,0,-(far+near)/(far-near),-1,
			0,0,-(2*far*near)/(far-near),0)
end

--[[function lookAt(eye, center, up)
	F = center - eye
	F:normalize()
	up:normalize()
	s = F*up
	u = s:normalize() * F
return mat4(
-- yet can't find good glu::LookAt documentation to finish this matrix
-- it would be nice if you can help me there ^^

)
end]]